from enum import StrEnum


class PriceCurrency(StrEnum):
    PLN = 'PLN'
    EUR = 'EUR'

    @staticmethod
    def to_common(value: str) -> 'PriceCurrency':
        value = value.upper()
        if value == 'PLN':
            return PriceCurrency.PLN
        
        if value == 'EUR':
            return PriceCurrency.EUR

        raise ValueError(f'Invalid value: {value}')


from typing import Protocol


class ResultWriterService(Protocol):
    def write_result(self, content: dict, key: str) -> None:
        pass
    def write_cache(self, content: dict, key: str, resolver_name: str)-> None:
        pass


import json
import os
from cs_ai_common.logging.internal_logger import InternalLogger
import re

from cs_ai_common.seed_data.seed_data_resolver import SeedDataResolver

class LambdaEventSeedDataResolver(SeedDataResolver):
    def __init__(self, event: dict):
        self.event = event

    def get_seed_data(self) -> dict:
        InternalLogger.LogDebug("Getting seed data from lambda event. {}".format(json.dumps(self.event)))        
        seed_data = self.event["seed_data"]
        task_id = self.event["task_id"]
        created_date = self.event["created_date"]
        return {
            "seed_data": {
                "Type": "osobowe",
                "Make": transform(seed_data["Make"]),
                "Model": seed_data["Model"],
                "ProductionYear": seed_data["ProductionYear"],
                "FuelType": seed_data["FuelType"],
                "Mileage": seed_data["Mileage"],
                "Capacity": seed_data["Capacity"],
                "Transmission": seed_data["Transmision"],
                "HorsePower": seed_data["HorsePower"],
                "Generation": ""
            },
            "task_id": task_id,
            "created_date": created_date
        }

def transform(make: str) -> str:
    _val = make.replace(" ", "")
    return re.sub(r'(?<!^)(?=[A-Z])', '-', _val).lower()

     


from typing import Dict, Optional

from cs_ai_common.resolver_cache.cache_key import build_s3_cache_key
from cs_ai_common.s3.resolver_results import write_result_to_s3
from cs_ai_common.s3.utils import try_get_object


def try_get_cached_result(cache_key: str, resolver_name: str, bucket_name: str) -> Dict | None:
    return try_get_object(build_s3_cache_key(cache_key, resolver_name), bucket_name)

def save_cache_result(cache_key: str, resolver_name: str, result: Dict, bucket_name: str) -> None:
    cache_key = build_s3_cache_key(cache_key, resolver_name)
    write_result_to_s3(result, cache_key, bucket_name)


from cs_ai_common.logging.internal_logger import InternalLogger
from fastapi import HTTPException, Request, status
from fastapi.responses import JSONResponse
from pydantic import ValidationError



async def catch_exceptions_middleware(request: Request, call_next):
    try:
        return await call_next(request)
    except ValueError as e:
        InternalLogger.LogError(f"ValueError: {str(e)}")
        return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST, content={"message": str(e)}
        )
    except ValidationError as e:
        InternalLogger.LogError(f"ValidationError: {str(e)}")
        return JSONResponse(
            status_code=status.HTTP_400_BAD_REQUEST, content={"message": str(e)}
        )
    except HTTPException as e:
        InternalLogger.LogError(f"HTTPException: {e.status_code} - {e.detail}")
        return JSONResponse(status_code=e.status_code, content={"message": e.detail})
    except Exception as e:
        InternalLogger.LogError(f"Exception: {str(e)}")
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content={"message": str(e)},
        )